
// PathFinder.java
// https://www.programcreek.com/2014/06/leetcode-word-search-java/
// Modified by Andrew Davison, ad@fivedots.coe.psu.ac.th, Dec. 2022

/* Load a grid of letters and try to cover it with words
   starting from the center of the square. Each letter can be used
   only once, and the next letter in a word can be North, South,
   East, or West of the current one. 

   A 9x9 grid is nearing the size limit. It takes 200-400 seconds
   to generate an answer, as the examples below illustrate.

   The quality of the chosen words are arguably not that great.
   Hunspell obviously contains a lof of Scrabble-acceptable 
   three-letter words, which I wouldn't classify as common
   words. I 'fixed' this problem in Anagrams.txt by using lists
   of common 2- and 3-letter words.

  Usage:
    > compile PathFinder.java
    > run PathFinder <data-file>
  e.g. 
    > run PathFinder wordSq1.txt
  
  ---------------------------------
      0  1  2  3  4  5  6  7  8
  
  0   l  w  e  k  a  w  o  n  s
  1   o  o  h  i  l  f  s  c  f
  2   v  e  c  m  n  e  y  a  r
  3   o  t  s  t  o  r  e  t  s
  4   o  o  d  m  f  r  h  t  u
  5   w  n  i  e  s  o  o  r  g
  6   y  d  w  t  t  b  t  m  t
  7   r  a  k  e  t  i  s  b  e
  8   o  n  a  r  t  e  n  e  r
  ---------------------------------
  Loaded Hunspell
  Starting search at (4,4)
  Grid covered with words
  Duration: 216 secs
  
  Path:
    frost -- (4,4)(6,4)  bit -- (6,5)(7,4)
    ten -- (8,4)(8,6)  storm -- (7,6)(6,7)
    beret -- (7,7)(6,8)  gust -- (5,8)(3,7)
    the -- (4,7)(3,6)  rot -- (3,5)(3,3)
    meter -- (4,3)(8,3)  anorak -- (8,2)(7,2)
    windy -- (6,2)(6,0)  wood -- (5,0)(4,2)
    stove -- (3,2)(2,1)  chimney -- (2,2)(2,6)
    scarf -- (1,6)(1,8)  snow -- (0,8)(0,5)
    flake -- (1,5)(0,2)  wool -- (0,1)(0,0)
  No. of words: 18

  ----------------
  > run PathFinder wordSq2.txt

  Loading wordSq2.txt
  
  ---------------------------------
      0  1  2  3  4  5  6  7  8
  
  0   i  c  t  e  r  o  w  d  a
  1   p  e  n  r  b  e  y  e  e
  2   e  e  o  b  p  a  t  e  h
  3   a  g  n  e  s  t  o  a  n
  4   t  s  t  e  c  e  l  s  o
  5   c  a  r  l  l  b  o  t  l
  6   r  n  i  t  o  t  t  p  o
  7   o  c  o  o  m  w  a  h  c
  8   s  i  f  b  m  o  s  e  s
  ---------------------------------
  
  Loaded Hunspell
  Starting search at (4,4)
  Grid covered with words
  Duration: 353 secs
  
  Path:
    cell -- (4,4)(5,4)  bot -- (5,5)(6,6)
    tom -- (6,5)(7,4)  womb -- (7,5)(8,3)
    foo -- (8,2)(7,3)  tin -- (6,3)(6,1)
    cis -- (7,1)(8,0)  orc -- (7,0)(5,0)
    art -- (5,1)(4,2)  stage -- (4,1)(2,1)
    epic -- (2,0)(0,1)  enter -- (1,1)(1,3)
    bone -- (2,3)(3,3)  spate -- (3,4)(4,5)
    lot -- (4,6)(2,6)  east -- (2,7)(5,7)
    phases -- (6,7)(8,8)  colon -- (7,8)(3,8)
    head -- (2,8)(0,7)  eye -- (1,7)(1,5)
    brow -- (1,4)(0,6)
  No. of words: 21

*/

import java.util.*;


public class PathFinder
{
  // limits on word sizes  3 <= word size <= 7
  private static final int MIN_LEN = 3;
  private static final int MAX_LEN = 7;

  private static final int[][] DIRS = {
            {1, 0}, {0, 1}, {-1, 0}, {0, -1} };
      // four directions in which a word can be extended 


  private static HunspellChecker hsc;
  private static ArrayList<GridWord> gWords;
           // the list of words that cover the grid
  private static int numUsedChars;  
          // used to determine when the entire grid is covered


  public static void main(String[] args)
  {
    if (args.length != 1) {
      System.out.println("Usage: run PathFinder <data-file>");
      return;
    }
    char[][] grid = WordUtils.loadGrid(args[0]);

    // small grids for debugging 
/*
    char[][] grid = new char[][]{
       {'a', 'c', 'g'},
       {'t', 'd', 'o'},
       {'b', 'a', 't'}};
*/
/*
    char[][] grid = new char[][]{
       {'t', 'a', 'b', 't', 'a'},
       {'m', 't', 'a', 'm', 'c'},
       {'a', 'n', 'd', 'o', 'g'},
       {'n', 'a', 'b', 'n', 'a'},
       {'c', 'a', 'n', 'b', 'e'}
    };
*/
    WordUtils.printGrid(grid);

    hsc = new HunspellChecker();

    gWords = new ArrayList<>();
    numUsedChars = 0;

    long start = System.currentTimeMillis();

    int center = grid.length/2;
    System.out.println("Starting search at (" + 
                                   center + "," + center + ")");

    boolean filledGrid = startWordAt(center,center, grid); 

    hsc.close();
    long duration = System.currentTimeMillis() - start;
    System.out.println("Duration: " + duration/1000 + " secs");

    if (!filledGrid)
      System.out.println("Unable to cover the grid with words");
    else {
      // print words
      System.out.println("\nPath:");
      WordUtils.printWords(gWords, 2);
    }
  } /* main */



  private static boolean startWordAt(int r, int c, char[][] grid)
  /* Start a new word at (r,c).
     As the grid is covered with words, the used characters are
     replaced by '#'s, which makes it easy to avoid using them
     twice. If a word is rejected, the '#' is turned back to its
     original character.
  */
  {
    if (numUsedChars == grid.length * grid[0].length) {
      System.out.println("Grid covered with words");
      return true;
    }

    if (!WordUtils.onGrid(grid, r, c))
      return false;
    if (grid[r][c] == '#')  // this letter has been used
      return false;


    // start building the word at (r,c)
    String word = "" + grid[r][c];
    char temp = grid[r][c];
    grid[r][c] = '#';
    numUsedChars++;

    // try extending this new word in all the possible directions
    for(int[] dir : DIRS) {
      if (extendWord(word, r, c, r, c, dir, grid))
        return true;
    }

    // could not extend this word, so undo grid change
    grid[r][c] = temp;
    numUsedChars--;
    return false;
  } /* startWordAt */



  private static boolean extendWord(String word, 
           int startR, int startC, int r, int c, 
           int[] dir, char[][] grid)
  /* extend the word that starts at (startR, startC)
     and currently ends at (r,c)
  */
  {
    int r1 = r + dir[0];  // x-axis direction
    int c1 = c + dir[1];  // y-axis

    if (!WordUtils.onGrid(grid, r1, c1))
      return false;
    if (grid[r1][c1] == '#') // this letter has been used
      return false;

    if (word.length() > MAX_LEN)  // word is too big
      return false; 

    // try adding the character at (r1,c1) to word
    word = word + grid[r1][c1];
    char temp = grid[r1][c1];
    grid[r1][c1] = '#';
    numUsedChars++;

    if ((word.length() >= MIN_LEN) && hsc.isCorrect(word)) {
      GridWord gw = new GridWord(word, startR, startC, r1, c1);
          // store the new word as a GridWord object
      //System.out.println("Added " + gw);
      gWords.add(gw);
	
      // and begin a new word at an adjacent cell
      for(int[] newDir : DIRS) {
        int r2 = r1 + newDir[0];
        int c2 = c1 + newDir[1];
        if (startWordAt(r2, c2, grid))
          return true;
      }
	
      /* if we reach here, then we failed to add 
         another word, so remove this one */
      GridWord removed = gWords.remove( gWords.size()-1);
      // System.out.println("Removed: " + removed.getWord());
    }

    // and resume trying to extend the word
    for(int[] newDir : DIRS) {
      if (extendWord(word, startR, startC, r1, c1, newDir, grid))
        return true;
    }

    /* if we reach here, then there was no way to
       extend this word, so undo the changes 
       to grid[] and this word, and give up
    */
    grid[r1][c1] = temp;
    numUsedChars--;
    word = word.substring(0, word.length() - 1);
    
    return false;
  } /* extendWord */


}  // end of PathFinder class